import path from 'node:path';
import { _load } from '../../util/loader.js';
import { getPackageName } from '../../util/modules.js';
import { isAbsolute } from '../../util/path.js';
import { timerify } from '../../util/performance.js';
import { hasDependency } from '../../util/plugin.js';
import { require } from '../../util/require.js';
export const NAME = 'Jest';
export const ENABLERS = ['jest'];
export const isEnabled = ({ dependencies }) => hasDependency(dependencies, ENABLERS);
export const CONFIG_FILE_PATTERNS = ['jest.config.{js,ts,mjs,cjs,json}'];
export const ENTRY_FILE_PATTERNS = [];
const join = (base, id) => (isAbsolute(id) ? id : path.join(path.dirname(base), id));
const resolveExtensibleConfig = async (configFilePath) => {
    const config = await _load(configFilePath);
    if (config?.preset) {
        const { preset } = config;
        if (preset.startsWith('.') || isAbsolute(preset)) {
            const presetConfigPath = join(configFilePath, preset);
            const presetConfig = await resolveExtensibleConfig(presetConfigPath);
            delete config.preset;
            Object.assign(config, presetConfig);
        }
    }
    return config;
};
const findJestDependencies = async (configFilePath, { cwd }) => {
    const config = await resolveExtensibleConfig(configFilePath);
    if (!config)
        return [];
    const dependencies = [];
    const entryFiles = [];
    const handleEntries = (name) => {
        name = name.includes('<rootDir>') ? path.join(cwd, name.replace(/^.*<rootDir>/, '')) : name;
        if (name.startsWith('.') || isAbsolute(name)) {
            entryFiles.push(require.resolve(join(configFilePath, name)));
        }
        else {
            dependencies.push(name);
        }
    };
    if (config.setupFiles)
        config.setupFiles.forEach(handleEntries);
    if (config.setupFilesAfterEnv)
        config.setupFilesAfterEnv.forEach(handleEntries);
    if (config.transform) {
        Object.values(config.transform)
            .map(transform => (typeof transform === 'string' ? transform : transform[0]))
            .forEach(handleEntries);
    }
    const presets = config.preset ? [config.preset] : [];
    const environments = config.testEnvironment === 'jsdom' ? ['jest-environment-jsdom'] : [];
    const resolvers = config.resolver ? [config.resolver] : [];
    const watchPlugins = config.watchPlugins?.map(watchPlugin => (typeof watchPlugin === 'string' ? watchPlugin : watchPlugin[0])) ?? [];
    return {
        dependencies: [...presets, ...environments, ...dependencies, ...resolvers, ...watchPlugins].map(getPackageName),
        entryFiles,
    };
};
export const findDependencies = timerify(findJestDependencies);
